﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using CSCore.Codecs;
using CSCore.Codecs.WAV;

namespace Decoding
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args == null || args.Length < 2)
            {
                Console.WriteLine("Invalid arguments: {inputfile} {outputfile.WAV}");
                Environment.Exit(-1);
            }

            try
            {
                using (var stream = CodecFactory.Instance.GetCodec(args[0]))
                {
                    using (var waveStream = new WaveWriter(File.OpenWrite(args[1]), stream.WaveFormat))
                    {
                        byte[] buffer = new byte[stream.WaveFormat.BytesPerSecond];
                        int read = 0;

                        while ((read = stream.Read(buffer, 0, buffer.Length)) > 0)
                        {
                            waveStream.Write(buffer, 0, read);
                        }
                    }
                }
            }
            catch (NotSupportedException e)
            {
                Console.WriteLine("Format nicht unterstützt: " + e.Message);
            }
            catch (Exception e)
            {
                Console.WriteLine("Unbekannter Fehler: " + e.Message);
            }

            Console.WriteLine("Decoding finished");
            Console.ReadKey();

            System.Diagnostics.Process.Start(args[1]);
        }
    }
}
