﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CSCore;
using CSCore.Codecs;
using CSCore.SoundOut;
using CSCore.SoundOut.DirectSound;
using CSCore.Streams.SampleConverter;
using System.Threading.Tasks;

namespace WPFVisualisation
{
    public partial class MainWindow : Window
    {
        ISoundOut _soundOut;

        public MainWindow()
        {
            InitializeComponent();
            visualizer.ItemsSource = CSCore.Visualisation.WPF.Point3DSpectrum1.DefaultColors;
        }

        private void OnOpenFile(object sender, RoutedEventArgs e)
        {
            var ofn = new Microsoft.Win32.OpenFileDialog()
            {
                Filter = CodecFactory.SupportedFilesFilterDE,
                Title = "Datei auswählen"
            };

            if (ofn.ShowDialog().GetValueOrDefault())
            {
                OpenSource(CodecFactory.Instance.GetCodec(ofn.FileName));
            }
        }

        private void OnOpenStream(object sender, RoutedEventArgs e)
        {
            StreamURLSelector streamSelector = new StreamURLSelector();
            if (streamSelector.ShowDialog().GetValueOrDefault())
            {
                var stream = new CSCore.Codecs.MP3.Mp3WebStream(streamSelector.Value, true);
                stream.ConnectionCreated += (s, args) =>
                {
                    if (args.Success)
                    {
                        Dispatcher.Invoke(new Action(() =>
                        {
                            OpenSource(stream);
                        }));
                    }
                    else MessageBox.Show("Es konnte keine Verbindung zum Server hergestellt werden.", "Fehler", MessageBoxButton.OK, MessageBoxImage.Error);
                };
            }
        }

        private void OpenSource(IWaveSource source)
        {
            Stop();

            //ausgehend davon, dass Treiber 16Bit PCM unterstützen. Sollte eigentlich standart sein. In der Theorie jedoch kann es auch sein, dass nur 8bit unterstützt werden und nur mono etc...
            source = new SampleToRaw16(RawToSampleBase.CreateConverter(source));

            visualizer.BaseStream = source;
            visualizergraph.BaseStream = visualizer.BaseStream;
            source = visualizergraph;

            _soundOut = new DirectSoundOut();
            _soundOut.Initialize(source);
            _soundOut.Play();
        }

        private void Stop()
        {
            if (_soundOut != null)
            {
                _soundOut.Stop();
                _soundOut.Dispose();
                _soundOut.WaveSource.Dispose();

                _soundOut = null;
            }
        }

        private void OnClosing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            Stop();
        }
    }
}
